package gregapi.tileentity;

import static gregapi.data.CS.FORGE_DIR;
import static gregapi.data.CS.RF_PER_EU;
import static gregapi.data.CS.RF_PER_MJ;
import gregapi.code.TagData;
import gregapi.data.TD;
import gregapi.util.UT;
import ic2.api.energy.tile.IEnergySink;
import net.minecraft.tileentity.TileEntity;
import cofh.api.energy.IEnergyConnection;
import cofh.api.energy.IEnergyHandler;
import cofh.api.energy.IEnergyReceiver;

/**
 * @author Gregorius Techneticies
 * 
 * For Internal Use.
 */
public class EnergyCompat {
	private static boolean RF_ENERGY = false, IC_ENERGY = false, CHECK_ALL = true;
	
	private static void checkAvailabilities() {
		if (CHECK_ALL) {
			try {
				IEnergyReceiver.class.getCanonicalName();
				IEnergyHandler.class.getCanonicalName();
				IEnergyConnection.class.getCanonicalName();
				RF_ENERGY = true;
			} catch(Throwable e) {/**/}
			try {
				IEnergySink.class.getCanonicalName();
				IC_ENERGY = true;
			} catch(Throwable e) {/**/}
			CHECK_ALL = false;
		}
	}
	
	public static long insertEnergyInto(TagData aEnergyType, byte aSideInto, long aSize, long aAmount, Object aEmitter, TileEntity aReceiver) {
		long rUsedAmount = 0;
		checkAvailabilities();
		if (IC_ENERGY && aEnergyType == TD.Energy.EU && aReceiver instanceof IEnergySink) {
    		if (((IEnergySink)aReceiver).acceptsEnergyFrom(aEmitter instanceof TileEntity ? (TileEntity)aEmitter : null, FORGE_DIR[aSideInto])) {
    			while (aAmount > rUsedAmount && ((IEnergySink)aReceiver).getDemandedEnergy() > 0 && ((IEnergySink)aReceiver).injectEnergy(FORGE_DIR[aSideInto], aSize, aSize) < aSize) rUsedAmount++;
    		}
		} else if (RF_ENERGY) {
			long tSizeToReceive = 0;
			// GT KineticUnits auto-convert to RF, but only in the Push Phase, so when they are postive!
			if (aEnergyType == TD.Energy.KU) tSizeToReceive = aSize*RF_PER_EU; else
			// MJ auto-convert to RF too. And yes I do know that BuildCraft and other Mods moved away from MJ to use RF.
			if (aEnergyType == TD.Energy.MJ) tSizeToReceive = aSize*RF_PER_MJ; else
			// RF is RF and therefore doesn't really need to be converted therefore 1:1 Ratio
			if (aEnergyType == TD.Energy.RF) tSizeToReceive = Math.abs(aSize);
			
			if (tSizeToReceive > 0) {
				if (aReceiver instanceof IEnergyReceiver) {
					if (!(aReceiver instanceof IEnergyConnection) || ((IEnergyConnection)aReceiver).canConnectEnergy(FORGE_DIR[aSideInto])) {
						rUsedAmount += UT.Code.units(((IEnergyReceiver)aReceiver).receiveEnergy(FORGE_DIR[aSideInto], (int)UT.Code.bind(0, Integer.MAX_VALUE, aAmount * tSizeToReceive), false), tSizeToReceive, 1, true);
					}
				} else if (aReceiver instanceof IEnergyHandler) {
					if (!(aReceiver instanceof IEnergyConnection) || ((IEnergyConnection)aReceiver).canConnectEnergy(FORGE_DIR[aSideInto])) {
						rUsedAmount += UT.Code.units(((IEnergyHandler)aReceiver).receiveEnergy(FORGE_DIR[aSideInto], (int)UT.Code.bind(0, Integer.MAX_VALUE, aAmount * tSizeToReceive), false), tSizeToReceive, 1, true);
					}
				}
			}
		}
		return rUsedAmount;
	}
}